@@ -1,26 +1,27 @@
 ;;; find-dired.el --- run a `find' command and dired the output
 
-;;; Copyright (C) 1992, 1994, 1995 Free Software Foundation, Inc.
+;; Copyright (C) 1992, 1994, 1995 Free Software Foundation, Inc.
 
 ;; Author: Roland McGrath <roland@gnu.ai.mit.edu>,
 ;;	   Sebastian Kremer <sk@thp.uni-koeln.de>
 ;; Keywords: unix
 
-;;; This program is free software; you can redistribute it and/or modify
-;;; it under the terms of the GNU General Public License as published by
-;;; the Free Software Foundation; either version 2, or (at your option)
-;;; any later version.
-;;;
-;;; This program is distributed in the hope that it will be useful,
-;;; but WITHOUT ANY WARRANTY; without even the implied warranty of
-;;; MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-;;; GNU General Public License for more details.
-;;;
-;;; A copy of the GNU General Public License can be obtained from this
-;;; program's author (send electronic mail to roland@ai.mit.edu) or from
-;;; the Free Software Foundation, Inc., 675 Mass Ave, Cambridge, MA
-;;; 02139, USA.
-;;;
+;; This file is part of GNU Emacs.
+
+;; GNU Emacs is free software; you can redistribute it and/or modify
+;; it under the terms of the GNU General Public License as published by
+;; the Free Software Foundation; either version 2, or (at your option)
+;; any later version.
+
+;; GNU Emacs is distributed in the hope that it will be useful,
+;; but WITHOUT ANY WARRANTY; without even the implied warranty of
+;; MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+;; GNU General Public License for more details.
+
+;; You should have received a copy of the GNU General Public License
+;; along with GNU Emacs; see the file COPYING.  If not, write to the
+;; Free Software Foundation, Inc., 59 Temple Place - Suite 330,
+;; Boston, MA 02111-1307, USA.
 
 ;;; Code:
 
@@ -37,7 +38,11 @@ gives the option (or options) to `find' that produce the desired output.
 LS-SWITCHES is a list of `ls' switches to tell dired how to parse the output.")
 
 ;;;###autoload
-(defvar find-grep-options (if (eq system-type 'berkeley-unix) "-s" "-q")
+(defvar find-grep-options
+  (if (or (eq system-type 'berkeley-unix)
+	  (string-match "solaris2" system-configuration)
+	  (string-match "irix" system-configuration))
+      "-s" "-q")
   "*Option to grep to be as silent as possible.
 On Berkeley systems, this is `-s'; on Posix, and with GNU grep, `-q' does it.
 On other systems, the closest you can come is to use `-l'.")
@@ -53,7 +58,10 @@ On other systems, the closest you can come is to use `-l'.")
   "Run `find' and go into dired-mode on a buffer of the output.
 The command run (after changing into DIR) is
 
-    find . \\( ARGS \\) -ls"
+    find . \\( ARGS \\) -ls
+
+except that the variable `find-ls-option' specifies what to use
+as the final argument."
   (interactive (list (read-file-name "Run find in directory: " nil "" t)
 		     (read-string "Run find (with args): " find-args
 				  '(find-args-history . 1))))
@@ -77,6 +85,10 @@ The command run (after changing into DIR) is
 		     (car find-ls-option)))
   ;; The next statement will bomb in classic dired (no optional arg allowed)
   (dired-mode dir (cdr find-ls-option))
+  ;; This really should rerun the find command, but I don't
+  ;; have time for that.
+  (use-local-map (append (make-sparse-keymap) (current-local-map)))
+  (define-key (current-local-map) "g" 'undefined)
   ;; Set subdir-alist so that Tree Dired will work:
   (if (fboundp 'dired-simple-subdir-alist)
       ;; will work even with nested dired format (dired-nstd.el,v 1.15
@@ -131,8 +143,11 @@ Thus ARG can also contain additional grep options."
   (interactive "DFind-grep (directory): \nsFind-grep (grep regexp): ")
   ;; find -exec doesn't allow shell i/o redirections in the command,
   ;; or we could use `grep -l >/dev/null'
+  ;; We use -type f, not ! -type d, to avoid getting screwed
+  ;; by FIFOs and devices.  I'm not sure what's best to do
+  ;; about symlinks, so as far as I know this is not wrong.
   (find-dired dir
-	      (concat "! -type d -exec grep " find-grep-options " "
+	      (concat "-type f -exec grep " find-grep-options " "
 		      args " {} \\\; ")))
 
 (defun find-dired-filter (proc string)
