@@ -1,6 +1,6 @@
 ;;; version.el --- record version number of Emacs.
 
-;;; Copyright (C) 1985, 1992, 1994 Free Software Foundation, Inc.
+;;; Copyright (C) 1985, 1992, 1994, 1995 Free Software Foundation, Inc.
 
 ;; Maintainer: FSF
 ;; Keywords: internal
@@ -18,29 +18,28 @@
 ;; GNU General Public License for more details.
 
 ;; You should have received a copy of the GNU General Public License
-;; along with GNU Emacs; see the file COPYING.  If not, write to
-;; the Free Software Foundation, 675 Mass Ave, Cambridge, MA 02139, USA.
+;; along with GNU Emacs; see the file COPYING.  If not, write to the
+;; Free Software Foundation, Inc., 59 Temple Place - Suite 330,
+;; Boston, MA 02111-1307, USA.
 
 ;;; Code:
 
-(defconst emacs-version "19.24" "\
+(defconst emacs-version "20.1" "\
 Version numbers of this version of Emacs.")
 
 (defconst emacs-major-version
   (progn (string-match "^[0-9]+" emacs-version)
-	 (string-to-int (substring emacs-version
-				   (match-beginning 0) (match-end 0))))
+	 (string-to-int (match-string 0 emacs-version)))
   "Major version number of this version of Emacs.
 This variable first existed in version 19.23.")
 
 (defconst emacs-minor-version
   (progn (string-match "^[0-9]+\\.\\([0-9]+\\)" emacs-version)
-	 (string-to-int (substring emacs-version
-				   (match-beginning 1) (match-end 1))))
+	 (string-to-int (match-string 1 emacs-version)))
   "Minor version number of this version of Emacs.
 This variable first existed in version 19.23.")
 
-(defconst emacs-build-time (current-time-string) "\
+(defconst emacs-build-time (current-time) "\
 Time at which Emacs was dumped out.")
 
 (defconst emacs-build-system (system-name))
@@ -52,21 +51,28 @@ Don't use this function in programs to choose actions according
 to the system configuration; look at `system-configuration' instead."
   (interactive "P")
   (let ((version-string 
-         (format "GNU Emacs %s of %s %s on %s (%s)"
+         (format (if (not (interactive-p))
+		     "GNU Emacs %s (%s%s)\n of %s on %s"
+		   "GNU Emacs %s (%s%s) of %s on %s")
                  emacs-version
-                 (substring emacs-build-time 0
-                            (string-match " *[0-9]*:" emacs-build-time))
-                 (substring emacs-build-time 
-                            (string-match "[0-9]*$" emacs-build-time))
-                 emacs-build-system system-configuration)))
+		 system-configuration
+		 (cond ((featurep 'motif) ", Motif")
+		       ((featurep 'x-toolkit) ", X toolkit")
+		       (t ""))
+		 (format-time-string "%a %b %e %Y" emacs-build-time)
+                 emacs-build-system)))
     (if here 
         (insert version-string)
       (if (interactive-p)
-          (message "%s" (emacs-version))
+          (message "%s" version-string)
         version-string))))
 
 ;;; We hope that this alias is easier for people to find.
-(fset 'version 'emacs-version)
+(defalias 'version 'emacs-version)
+
+;;; We put version info into the executable in the form that UNIX what(1) uses.
+(or (memq system-type '(vax-vms windows-nt ms-dos))
+    (purecopy (concat "\n@(#)" (emacs-version) "\n")))
 
 ;;Local variables:
 ;;version-control: never
