@@ -174,10 +174,10 @@ extern Lisp_Object Vlucid_menu_bar_dirty_flag;
 extern Lisp_Object Qrecompute_lucid_menubar, Qactivate_menubar_hook;
 
 /* Total number of times read_char has returned.  */
-extern int num_input_chars;
+extern int num_input_events;
 
 /* Total number of times read_char has returned, outside of macros.  */
-extern int num_nonmacro_input_chars;
+extern int num_nonmacro_input_events;
 
 /* Nonzero means polling for input is temporarily suppressed.  */
 extern int poll_suppress_count;
@@ -196,15 +196,12 @@ extern Lisp_Object Vfunction_key_map;
 extern Lisp_Object this_command_keys;
 extern int this_command_key_count;
 
-#ifdef MULTI_FRAME
 /* The frame in which the last input event occurred, or Qmacro if the
    last event came from a macro.  We use this to determine when to
    generate switch-frame events.  This may be cleared by functions
    like Fselect_frame, to make sure that a switch-frame event is
    generated by the next character.  */
 extern Lisp_Object internal_last_event_frame;
-#endif
-
 
 /* Macros for dealing with lispy events.  */
 
