@@ -5,7 +5,7 @@ This file is part of GNU Emacs.
 
 GNU Emacs is free software; you can redistribute it and/or modify
 it under the terms of the GNU General Public License as published by
-the Free Software Foundation; either version 1, or (at your option)
+the Free Software Foundation; either version 2, or (at your option)
 any later version.
 
 GNU Emacs is distributed in the hope that it will be useful,
@@ -15,7 +15,8 @@ GNU General Public License for more details.
 
 You should have received a copy of the GNU General Public License
 along with GNU Emacs; see the file COPYING.  If not, write to
-the Free Software Foundation, 675 Mass Ave, Cambridge, MA 02139, USA.  */
+the Free Software Foundation, Inc., 59 Temple Place - Suite 330,
+Boston, MA 02111-1307, USA.  */
 
 /* The following line tells the configuration script what sort of 
    operating system this machine is likely to run.
@@ -30,19 +31,10 @@ the Free Software Foundation, 675 Mass Ave, Cambridge, MA 02139, USA.  */
 /* #define ncl_el	/* DPX/2 210,220 etc */
 /* #define ncl_mr 1	/* DPX/2 320,340 (and 360,380 ?) */
 
-/* The following three symbols give information on
- the size of various data types.  */
-
-#define SHORTBITS 16		/* Number of bits in a short */
-
-#define INTBITS 32		/* Number of bits in an int */
-
-#define LONGBITS 32		/* Number of bits in a long */
-
-/* Define BIG_ENDIAN iff lowest-numbered byte in a word
+/* Define WORDS_BIG_ENDIAN iff lowest-numbered byte in a word
    is the most significant byte.  */
 
-#define BIG_ENDIAN
+#define WORDS_BIG_ENDIAN
 
 /* Define NO_ARG_ARRAY if you cannot take the address of the first of a
  * group of arguments and treat it as an array of the arguments.  */
@@ -195,16 +187,6 @@ the Free Software Foundation, 675 Mass Ave, Cambridge, MA 02139, USA.  */
 */
 #define GETPGRP_NO_ARG
 
-/* 
- * X support _needs_ this
- */
-#define HAVE_SELECT
-/*
- * and select requires these
- */
-#define HAVE_TIMEVAL
-#define USE_UTIME
-
 /* select also needs this header file--but not in ymakefile.  */
 #ifndef NOT_C_CODE
 #include <sys/types.h>
@@ -231,6 +213,11 @@ the Free Software Foundation, 675 Mass Ave, Cambridge, MA 02139, USA.  */
 /* It doesn't seem we have sigpause */
 #undef HAVE_SYSV_SIGPAUSE
 
+#define POSIX_SIGNALS
+
+/* We don't need the definition from usg5-3.h with POSIX_SIGNALS.  */
+#undef sigsetmask
+
 
 /* on bos2.00.45 there is a bug that makes the F_SETOWN fcntl() call
    enters in an infinite loop. Avoid calling it  */
@@ -245,6 +232,9 @@ the Free Software Foundation, 675 Mass Ave, Cambridge, MA 02139, USA.  */
 /* Send signals to subprocesses by "typing" signal chars at them.  */
 #define SIGNALS_VIA_CHARACTERS
 
+/* This is to prevent memory clobbering on the DPX/2 200.  */
+#define LD_SWITCH_MACHINE -N -T32
+
  /* end of dpx2.h */
 
 
